@@ -4,24 +4,33 @@ use strict;
 use warnings;
 use UNIVERSAL::require;
 use YAML::Syck;
-our $VERSION='0.48';
+our $VERSION='0.62';
+
+use File::Copy;
 
 sub new{
   my ($class, $r,$prefix)=@_;
   my $self = {};
   $self->{request}= $r;
   $self->{submitted} = 0;
+  $self->{after_upload}='';
   $self->{submit_value} = '';
   $self->{count}=0;
   $self->{submit_id} = -1;
   $self->{addition_modules}='';
   $self->{prefix}='';
+  $self->{row_wrapper}='';
   $self->{prefix}=$prefix if ($prefix);
 
   bless( $self, $class );
   return $self;
 }
 
+sub set_row_wrapper{
+  my ($self, $wrapper) = @_;
+   $self->{row_wrapper}=$wrapper;
+}
+
 sub add_modules{
   my ($self, $mods) = @_;
    $self->{addition_modules}=$mods;
@@ -29,7 +38,6 @@ sub add_modules{
 
 sub add_constraint{
   my ($self, $params) = @_;
-
   my $name= $self->{prefix}.$params->{name};
   $params->{request}=$self->{request};
   my $class_name = "HTML::TurboForm::Constraint::" . $params->{ type };
@@ -42,6 +50,45 @@ sub add_uploads{
   $self->{uploads} = $uploads;
 }
 
+sub build_form{
+    my ($self, $data, $resultsource, $options)=@_;
+    
+    my @columns=$resultsource->columns;
+        
+    foreach (@columns){
+        my $forbidden=0;
+        my $info=$resultsource->column_info($_);
+        
+        my $label=$_;        
+        $label=$info->{label} if $info->{label};        
+        my $type='Text';
+        $type=$info->{fieldtype} if $info->{fieldtype};        
+        my $args={ type=>$type, name=> $_, label=> $label };        
+        if ($data->{$_}) {
+           while(my($key, $value) = each(%{$data->{$_}})){
+               $args->{$key}=$value if ($key ne 'name');
+           }
+        }        
+        my $k=$_;
+        if ($options->{definedonly}){
+        if ($options->{definedonly} eq '1'){
+        } else{
+           my $number =    keys %$info;    
+           $forbidden=1  if ($number==0);
+        }
+        } else{
+          my $number =    keys %$info;    
+           $forbidden=1  if ($number==0);          
+        }
+        
+        if (($data->{forbidden})&&($forbidden==0)){
+        #if ($data->{forbidden}){
+            foreach (@{$data->{forbidden}}){ $forbidden=1 if ($_ eq $k); }
+        }
+        $self->add_element($args) if $forbidden == 0;      
+    }
+}
+
 sub load{
     my ($self,$fn)=@_;
     my $data = LoadFile($fn);
@@ -112,6 +159,11 @@ sub add_element{
 
   my $class_name = "HTML::TurboForm::Element::" . $params->{ type };
   $class_name->require() or die "Class '" . $class_name . "' does not exist: $@";
+  
+  if (!$params->{wrapper}){  
+      $params->{wrapper}=$self->{row_wrapper} if ($self->{row_wrapper} ne '');
+  }
+  
   my $element= $class_name->new($params,$self->{uploads}->{$name.'_upload'});
 
   my $new_len =  push(@ { $self->{element} },  $element);
@@ -120,6 +172,13 @@ sub add_element{
   $self->{element_index}->{$name}->{frozen}=0;
   $self->{element_index}->{$name}->{ignore}='false';
   $self->{element_index}->{$name}->{error_message}='';
+  
+  if ($params->{type} eq 'Imageupload') {    
+    if ( exists $self->{uploads}->{$name."_upload"} ){   
+        $self->{after_upload}=$name;        
+        $element->do_img();   
+    }
+  }     
 
   if ($params->{type} eq 'Submit') {
     if (( exists $self->{request}->{$name.".x"} )or(exists $self->{request}->{$name})){
@@ -135,7 +194,7 @@ sub add_element{
     }
   }
 
-  if ($params->{type} eq 'Image') {
+  if (($params->{type} eq 'Image')||($params->{type} eq 'Upload')) {
     if ( exists $self->{request}->{$name.'_submit' } ){
       $self->{submitted}=1 ;
       $self->{submit_value} = $namew.'_uploaded';
@@ -170,11 +229,20 @@ sub add_element{
     }
   }
 
-  if ($params->{type} eq 'Captcha') {
+  if ($params->{type} eq 'Captcha') {      
+      my $tlabel=$params->{label1};
+      my $tlabel2=$params->{label2};
       my $tname=$name."_input";
-      my $c_val = $self->get_value($name);
+      my $tname2=$name."_input2";
       $self->add_element({ type => 'Text',  name => $tname } );
+      $self->add_element({ type => 'Text',  name => $tname2, class=>"form_input2" } );
+      my $c_val = $self->get_value($tname2);
+      
+      #use Data::Dumper;
+      #print STDERR Dumper($params);
       $self->add_constraint({ type=> 'Equation', operator=>'eq', name=>$tname, comp=>$c_val, text=>$params->{message} });
+      #$self->add_constraint({ type=> 'Equation', operator=>'eq', name=>$tname2, comp=>'', text=>$params->{message} });      
+      $self->add_constraint({ type=> 'Mintime', name=>$tname, keyname=> $params->{keyname}."2", keyphrase=>$params->{keyphrase} ,session=> $params->{session} , text=>'Error, please wait 5 Seconds and resubmit the form.' });                      
   }
 }
 
@@ -193,8 +261,8 @@ sub find_action{
 }
 
 sub do{
-  my ($self, $name, $fn)=@_;
-  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->$fn();
+  my ($self, $name, $fn,@args)=@_;
+  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->$fn(@args);
 }
 
 sub get_javascript{
@@ -275,7 +343,7 @@ sub render{
 
   my $table=-1;
   my $count=0;
-
+  $view='' if (!$view);
   $action=' action="'.$action.'" ' if ($action);
   $action='' if (!$action);
   my $table_class='class="form_table"';
@@ -283,42 +351,41 @@ sub render{
   $table_class=$self->{table_attibutes} if ($self->{table_attibutes});
 
     my $result='<form method=post '.$action.'enctype="multipart/form-data">';
+    
     if ($view eq 'table'){ $result.='<table '.$table_class.'>'; }
 
     foreach my $item(@{$self->{element}}) {
     my $name = $item->name;
 
     if ($self->{element_index}->{$name}->{ignore} ne 'true'){
-      $item->{table}=-1;
-
-      if ($view eq 'flat'){
-          if ($item->type ne 'Submit'){
-              my $label = $item->get_label();
-              my $value = $item->get_value();
-              $result.='<span class="form_label">'.$label."</span>: ".$value."<br />";
-          }
+        $item->{table}=-1;
+       
+        if ($view eq 'flat'){
+            if ($item->type ne 'Submit'){
+                my $label = $item->get_label();
+                my $value = $item->get_value();
+                $result.='<span class="form_label">'.$label."</span>: ".$value."<br />";
+            }
+        } else {
+            if ($item->type eq "TableEnd") {
+                $item->{table}=-1;
+                $table=-1;
+            }
+            if ($item->type eq "Table") {
+                $item->{table}=$item->columns;
+                $item->{colcount}=-1;
+                $count=-1;
+                $table=$item->columns;
+            }
+            if ($table>-1) {
+                $count++;
+                $count=1 if ($count>$table);
+                $item->{colcount}=$count;
+                $item->{table}=$table;
+            }
+            $result .= $item->render($self->{element_index}->{$name}, $view);
+         }
       } else {
-          if ($item->type eq "TableEnd") {
-             $item->{table}=-1;
-             $table=-1;
-          }
-          if ($item->type eq "Table") {
-             $item->{table}=$item->columns;
-             $item->{colcount}=-1;
-             $count=-1;
-             $table=$item->columns;
-          }
-          if ($table>-1) {
-             $count++;
-             $count=1 if ($count>$table);
-             $item->{colcount}=$count;
-             $item->{table}=$table;
-          }
-          $result .= $item->render($self->{element_index}->{$name}, $view);
-      }
-
-      }
-  else {
      $result.="<input type='hidden' name='$name' value='".$item->get_value()."'>";
   }
 }
@@ -328,6 +395,14 @@ sub render{
   return $result.'</form>';
 }
 
+
+sub uploaded{
+  my ($self) = @_;
+  
+  return $self->{after_upload} if ($self->{after_upload} ne '');    
+  return '';
+}
+
 sub submit{
   my ($self) = @_;
   my $result='';
@@ -344,7 +419,6 @@ sub submitted{
   if ($self->{submit_value} ne '') {
     $result=$self->{submit_value};
     #$result=substr($result,length($self->{prefix})) if ($self->{prefix} ne'');
-
     foreach my $item(@{$self->{constraints}}) {
       my $name=$item->{name};
       if ($item->check() == 0){
